package net.phbwt.paperwork.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.SystemBarStyle
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.ui.graphics.toArgb
import androidx.core.splashscreen.SplashScreen.Companion.installSplashScreen
import dagger.hilt.android.AndroidEntryPoint
import net.phbwt.paperwork.ui.main.MainScreen
import net.phbwt.paperwork.ui.theme.AppTheme
import net.phbwt.paperwork.ui.theme.DarkColors
import net.phbwt.paperwork.ui.theme.LightColors


@AndroidEntryPoint
class MainActivity : ComponentActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        installSplashScreen()
        super.onCreate(savedInstanceState)

        setContent {
            val isDark = isSystemInDarkTheme()
            ChangeSystemBarsTheme(isDark)
            AppTheme(isDark) {
                MainScreen()
            }
        }
    }

    @Composable
    private fun ChangeSystemBarsTheme(isDark: Boolean) {
        LaunchedEffect(isDark) {
            val statusbarDark = DarkColors.primaryContainer.toArgb()
            val statusbarLight = LightColors.primaryContainer.toArgb()
            // ignored on 29+
            val navbarLight = LightColors.primaryContainer.copy(alpha = .6f).toArgb()
            val navbarDark = DarkColors.primaryContainer.copy(alpha = .6f).toArgb()
            if (isDark) {
                enableEdgeToEdge(
                    statusBarStyle = SystemBarStyle.dark(statusbarDark),
                    navigationBarStyle = SystemBarStyle.auto(navbarLight, navbarDark),
                )
            } else {
                enableEdgeToEdge(
                    statusBarStyle = SystemBarStyle.light(statusbarLight, statusbarDark),
                    navigationBarStyle = SystemBarStyle.auto(navbarLight, navbarDark),
                )
            }
        }
    }
}


